// See below for required definitions 
// \Arduino15\packages\Seeeduino\tools\CMSIS-Atmel\1.2.1\CMSIS-Atmel\CMSIS\Device\ATMEL\samd21\include\samd21g18a.h
// \                                                                                                  \component\nvmxtrl.h
// https://ww1.microchip.com/downloads/en/DeviceDoc/SAM_D21_DA1_Family_DataSheet_DS40001882F.pdf

#define NVM_MEMORY ((volatile uint16_t *)FLASH_ADDR)
uint32_t buffer[32];

void setup() {
  Serial.begin(115200);
  while(!Serial) {delay(1); }
}

void loop() {
  for(int i = 0; i <= 7; i++) {
    buffer[i] = NVM_MEMORY[(NVMCTRL_USER / 2) + i];     // 16bit access 
  }
  Serial.println("AUX0  NVM User Row Mapping (Table 10-4.)");
  Serial.println();
  Serial.println("             +E   +C   +A   +8   +6   +4   +2   +0");
  Serial.print("0x"); Serial.print(NVMCTRL_USER, HEX); Serial.print(" : ");
  for(int i = 7; i >= 0; i--) {
    padZero(buffer[i]);    
    Serial.print(buffer[i], HEX );
    Serial.print(" ");       
  }
  Serial.println();
  Serial.println();     
  Serial.println("Boot Loader Size (Table 22-2.)");  
  int bp = buffer[0] & NVMCTRL_FUSES_BOOTPROT_Msk;	// 0x7  BOOTPROT{2:0]
  int bpSize;											                  // bootloader protection bytes size

  if(bp == 7) {
    bpSize = 0;
  }
  else {
    bpSize = 512 * pow(2, 6 - bp);
  }
  
  Serial.print("BOOTPROT = ");
  Serial.print(bp);
  Serial.print("   Bootloader Size = ");
  Serial.print(bpSize);
  Serial.println(" bytes");
  Serial.println();
  Serial.println();
     
  while(!Serial.available()>0){;}
  while(Serial.available()>0){Serial.read();}
 
}

//Padding ZERO
void padZero(int data) {
  if(data < 0x1000) {
    Serial.print("0");
  }
  if(data < 0x100) {
    Serial.print("0");
  }
  if(data < 0x10) {
    Serial.print("0");
  }
}
